﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using Quartz;
using XF.Domain.Queues;

namespace DataSharing.SendTask
{
    public class TaskExecuteJob : IJob, IDisposable
    {
        private readonly IQueue _queue;
        private readonly ILogger<TaskExecuteJob> _logger;

        public TaskExecuteJob(
            IQueue queue,
            ILogger<TaskExecuteJob> logger)
        {
            _queue = queue;
            _logger = logger;
        }

        public async Task Execute(IJobExecutionContext context)
        {
            var task = await _queue.DequeueAsync<DsSendTask>(SendTaskDefaults.SendTaskQueue, context.CancellationToken);
            if (task is not null)
            {
                _logger.LogInformation($"send task execute, taskId: {task.Id}");
            }
        }

        public void Dispose()
        {
            _logger.LogInformation($"execute job disposing");
        }
    }
}